@@ -1,4 +1,4 @@
-<TeXmacs|1.99.9>
+<TeXmacs|1.99.13>
 
 <style|<tuple|source|std>>
 
@@ -34,7 +34,7 @@
 
   <assign|made-by-TeXmacs|<macro|<float|footnote||<with|font-size|0.84|par-mode|justify|par-left|0cm|par-right|0cm|<active*|<move|<image|local:$TEXMACS_PATH/misc/images/tm_gnu3.ps||1fn||>|0fn|-0.2fn>><space|2spc><localize|This
   document has been produced using> GNU <TeXmacs> (<localize|see>
-  <with|font-family|tt|http://www.texmacs.org>).<right-flush>>>>>
+  <with|font-family|tt|https://www.texmacs.org>).<right-flush>>>>>
 
   <assign|tm-made|<macro|<with|ornament-color||ornament-shadow-color|<value|color>|ornament-sunny-color|<value|color>|ornament-border|1ln|ornament-shape|rounded|ornament-hpadding|0.5spc|ornament-vpadding|0.5spc|<ornament|<math|\<Backsigma\>>>>>>
 
@@ -383,11 +383,11 @@
 
   <assign|extend-right|<macro|body|right|<resize|<arg|body>|||<maximum|1r|<arg|right>>|>>>
 
-  <assign|swell|<macro|body|<resize|<arg|body>||<minimum|1b|1fnbot>||<maximum|1t|1fntop>>>>
+  <assign|inflate|<macro|body|<resize|<arg|body>||<minimum|1b|1fnbot>||<maximum|1t|1fntop>>>>
 
-  <assign|swell-bottom|<macro|body|<resize|<arg|body>||<minimum|1b|1fnbot>||>>>
+  <assign|inflate-bottom|<macro|body|<resize|<arg|body>||<minimum|1b|1fnbot>||>>>
 
-  <assign|swell-top|<macro|body|<resize|<arg|body>||||<maximum|1t|1fntop>>>>
+  <assign|inflate-top|<macro|body|<resize|<arg|body>||||<maximum|1t|1fntop>>>>
 
   \;
 
@@ -430,7 +430,7 @@
 
   <assign|stressed-distance|0.2fn>
 
-  <assign|stressed|<macro|body|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<with|color|<value|stressed-color>|<repeat*|<with|color|<unquote|<value|color>>|<arg|x>>|<move|<resize|-|<plus|0.6667l|0.3333r>||<plus|0.3333l|0.6667r>|>||<minus|-0.5ex|<value|stressed-distance>>>>>>|<arg|body>>>>>>
+  <assign|stressed|<macro|body|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<with|color|<unquote|<value|stressed-color>>|<repeat*|<with|color|<unquote|<value|color>>|<arg|x>>|<move|<resize|-|<plus|0.6667l|0.3333r>||<plus|0.3333l|0.6667r>|>||<minus|-0.5ex|<unquote|<value|stressed-distance>>>>>>>|<arg|body>>>>>>
 
   <drd-props|stressed|with-like|yes|arity|1|accessible|all>
 
@@ -474,19 +474,24 @@
 
   <assign|marked-padding|0.2fn>
 
-  <assign|marked|<macro|body|<style-with|src-compact|none|<datoms|<macro|x|<resize|<tabular|<tformat|<cwith|1|1|1|1|cell-background|<value|marked-color>>|<cwith|1|1|1|1|cell-lsep|<value|marked-padding>>|<cwith|1|1|1|1|cell-rsep|<value|marked-padding>>|<cwith|1|1|1|1|cell-bsep|<value|marked-padding>>|<cwith|1|1|1|1|cell-tsep|<value|marked-padding>>|<table|<row|<cell|<arg|x>>>>>>|<plus|1l|<value|marked-padding>>|<plus|1b|<value|marked-padding>>|<minus|1r|<value|marked-padding>>|<minus|1t|<value|marked-padding>>>>|<arg|body>>>>>
+  <assign|marked|<macro|body|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<resize|<tabular|<tformat|<cwith|1|1|1|1|cell-background|<unquote|<value|marked-color>>>|<cwith|1|1|1|1|cell-lsep|<unquote|<value|marked-padding>>>|<cwith|1|1|1|1|cell-rsep|<unquote|<value|marked-padding>>>|<cwith|1|1|1|1|cell-bsep|<unquote|<value|marked-padding>>>|<cwith|1|1|1|1|cell-tsep|<unquote|<value|marked-padding>>>|<table|<row|<cell|<arg|x>>>>>>|<plus|1l|<unquote|<value|marked-padding>>>|<plus|1b|<unquote|<value|marked-padding>>>|<minus|1r|<unquote|<value|marked-padding>>>|<minus|1t|<unquote|<value|marked-padding>>>>>|<arg|body>>>>>>
 
   <drd-props|marked|with-like|yes|arity|1|accessible|all>
 
   \;
 
-  <assign|todo|<macro|body|<with|color|dark
-  red|<style-with|src-compact|none|<datoms|<macro|x|<resize|<tabular|<tformat|<cwith|1|1|1|1|cell-background|pastel
-  red>|<cwith|1|1|1|1|cell-lsep|0fn>|<cwith|1|1|1|1|cell-rsep|0fn>|<cwith|1|1|1|1|cell-bsep|<value|marked-padding>>|<cwith|1|1|1|1|cell-tsep|<value|marked-padding>>|<table|<row|<cell|<arg|x>>>>>>|<plus|1l|0fn>|<plus|1b|<value|marked-padding>>|<minus|1r|0fn>|<minus|1t|<value|marked-padding>>>>|[<arg|body>]>>>>>
+  <assign|todo-color|dark red>
 
-  <assign|todo*|<macro|body|<with|color|dark
-  orange|<style-with|src-compact|none|<datoms|<macro|x|<resize|<tabular|<tformat|<cwith|1|1|1|1|cell-background|pastel
-  orange>|<cwith|1|1|1|1|cell-lsep|0fn>|<cwith|1|1|1|1|cell-rsep|0fn>|<cwith|1|1|1|1|cell-bsep|<value|marked-padding>>|<cwith|1|1|1|1|cell-tsep|<value|marked-padding>>|<table|<row|<cell|<arg|x>>>>>>|<plus|1l|0fn>|<plus|1b|<value|marked-padding>>|<minus|1r|0fn>|<minus|1t|<value|marked-padding>>>>|[<arg|body>]>>>>>
+  <assign|todo-bg-color|pastel red>
+
+  <assign|render-todo|<macro|fg|bg|body|<with|color|<arg|fg>|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<resize|<tabular|<tformat|<cwith|1|1|1|1|cell-background|<unquote|<arg|bg>>>|<cwith|1|1|1|1|cell-lsep|0fn>|<cwith|1|1|1|1|cell-rsep|0fn>|<cwith|1|1|1|1|cell-bsep|<unquote|<value|marked-padding>>>|<cwith|1|1|1|1|cell-tsep|<unquote|<value|marked-padding>>>|<table|<row|<cell|<arg|x>>>>>>|<plus|1l|0fn>|<plus|1b|<unquote|<value|marked-padding>>>|<minus|1r|0fn>|<minus|1t|<unquote|<value|marked-padding>>>>>|[<arg|body>]>>>>>>
+
+  <assign|todo|<macro|body|<render-todo|<value|todo-color>|<value|todo-bg-color>|<arg|body>>>>
+
+  <assign|todo*|<macro|body|<render-todo|dark orange|pastel
+  orange|<arg|body>>>>
+
+  <drd-props|render-todo|arity|3|color|0|color|1|accessible|2>
 
   <\active*>
     <\src-comment>
@@ -578,6 +583,8 @@
 
   <assign|extended-factor|1.2>
 
+  <assign|monospaced-factor|0.75>
+
   <assign|degraded-threshold|0.667>
 
   <assign|degraded-frequency|1.0>
@@ -606,12 +613,32 @@
 
   <assign|extended|<macro|body|<add-font-effect|hextended|<value|extended-factor>|<arg|body>>>>
 
+  <assign|monospaced|<macro|body|<add-font-effect|mono|<value|monospaced-factor>|<arg|body>>>>
+
   <assign|degraded|<macro|body|<add-font-effect|degraded|<merge|<value|degraded-threshold>|;|<value|degraded-frequency>>|<arg|body>>>>
 
   <assign|distorted|<macro|body|<add-font-effect|distorted|<merge|<value|distorted-strength>|;|<value|distorted-frequency>>|<arg|body>>>>
 
   <assign|gnawed|<macro|body|<add-font-effect|gnawed|<merge|<value|gnawed-strength>|;|<value|gnawed-frequency>>|<arg|body>>>>
 
+  <\active*>
+    <\src-comment>
+      Artistic font effects.
+    </src-comment>
+  </active*>
+
+  <assign|blurred-radius|0.6pt>
+
+  <assign|shadow-font-color|#222>
+
+  <assign|sunny-font-color|#ddd>
+
+  \;
+
+  <assign|blurred|<macro|body|<add-font-effect|blurred|<value|blurred-radius>|<arg|body>>>>
+
+  <assign|enhanced|<macro|body|<add-font-effect|enhanced|<merge|<value|blurred-radius>|;|<value|shadow-font-color>|;|<value|sunny-font-color>>|<arg|body>>>>
+
   <\active*>
     <\src-comment>
       Miscellaneous.
