@@ -12,7 +12,8 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (texmacs-module (generic insert-menu)
-  (:use (generic generic-edit)
+  (:use (utils edit selections)
+	(generic generic-edit)
 	(generic format-edit)
 	(generic format-geometry-edit)))
 
@@ -21,54 +22,59 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind insert-link-menu
-  ("Label" (make-label))
-  ("Reference" (make 'reference))
-  ("Page reference" (make 'pageref))
+  (when (not (selection-active-non-small?))
+    ("Label" (make-label))
+    ("Reference" (make 'reference))
+    ("Page reference" (make 'pageref)))
   ---
-  (if (detailed-menus?)
-      ("Include" (choose-file make-include "Include file" "")))
-  ("Hyperlink" (make 'hlink))
-  (if (detailed-menus?)
-      ("Action" (make 'action)))
+  (when (not (selection-active?))
+    (if (detailed-menus?)
+        ("Include" (choose-file make-include "Include file" ""))))
+  (when (not (selection-active-non-small?))
+    ("Link to URL" (make 'slink))
+    ("Hyperlink" (make 'hlink))
+    (if (detailed-menus?)
+        ("Action" (make 'action))))
   (if (simple-menus?)
-      ("Footnote" (make 'footnote)))
+      ("Footnote" (make-wrapped 'footnote)))
   (if (and (style-has? "std-dtd") (in-text?))
       ---
-      (-> "Citation"
-	  (if (not (style-has? "cite-author-year-dtd"))
-	      ("Visible" (make 'cite))
-	      ("Invisible" (make 'nocite))
-	      ("Detailed" (make 'cite-detail)))
-	  (if (style-has? "cite-author-year-dtd")
-	      (group "Abbreviated authors")
-	      ("Raw" (make 'cite-raw))
-	      ("Textual" (make 'cite-textual))
-	      ("Parenthesized" (make 'cite-parenthesized))
-	      ---
-	      (group "Full author list")
-	      ("Raw" (make 'cite-raw*))
-	      ("Textual" (make 'cite-textual*))
-	      ("Parenthesized" (make 'cite-parenthesized*))
-	      ---
-	      (group "Decomposed")
-	      ("Parenthesis" (make 'render-cite))
-	      ("Abreviated authors" (make 'cite-author-link))
-	      ("Full author list" (make 'cite-author*-link))
-	      ("Year" (make 'cite-year-link))
-	      ("Invisible" (make 'nocite))))
-      (-> "Index entry"
-	  ("Main" (make 'index))
-	  ("Sub" (make 'subindex))
-	  ("Subsub" (make 'subsubindex))
-	  ("Complex" (make 'index-complex))
-	  ---
-	  ("Interjection" (make 'index-line)))
-      (-> "Glossary entry"
-	  ("Regular" (make 'glossary))
-	  ("Explained" (make 'glossary-explain))
-	  ("Duplicate" (make 'glossary-dup))
-	  ---
-	  ("Interjection" (make 'glossary-line)))
+      (when (not (selection-active-non-small?))
+        (-> "Citation"
+            (if (not (style-has? "cite-author-year-dtd"))
+                ("Visible" (make 'cite))
+                ("Invisible" (make 'nocite))
+                ("Detailed" (make 'cite-detail)))
+            (if (style-has? "cite-author-year-dtd")
+                (group "Abbreviated authors")
+                ("Raw" (make 'cite-raw))
+                ("Textual" (make 'cite-textual))
+                ("Parenthesized" (make 'cite-parenthesized))
+                ---
+                (group "Full author list")
+                ("Raw" (make 'cite-raw*))
+                ("Textual" (make 'cite-textual*))
+                ("Parenthesized" (make 'cite-parenthesized*))
+                ---
+                (group "Decomposed")
+                ("Parenthesis" (make 'render-cite))
+                ("Abreviated authors" (make 'cite-author-link))
+                ("Full author list" (make 'cite-author*-link))
+                ("Year" (make 'cite-year-link))
+                ("Invisible" (make 'nocite))))
+        (-> "Index entry"
+            ("Main" (make 'index))
+            ("Sub" (make 'subindex))
+            ("Subsub" (make 'subsubindex))
+            ("Complex" (make 'index-complex))
+            ---
+            ("Interjection" (make 'index-line)))
+        (-> "Glossary entry"
+            ("Regular" (make 'glossary))
+            ("Explained" (make 'glossary-explain))
+            ("Duplicate" (make 'glossary-dup))
+            ---
+            ("Interjection" (make 'glossary-line))))
       (-> "Alternate"
           ("Bibliography"
            (make-alternate "Name of bibliography" "bib" 'with-bib))
@@ -81,7 +87,14 @@
           ("List of figures"
            (make-alternate "Name of list of figures" "figure" 'with-figure-list))
           ("List of tables"
-           (make-alternate "Name of list of tables" "table" 'with-table-list))))
+           (make-alternate "Name of list of tables" "table" 'with-table-list)))
+      ---
+      ("Reference to note" (make-note-ref))
+      (-> "Text for note"
+          ("Inline" (make-note-inline))
+          ("Wide" (make-note-wide))
+          (when (in-main-flow?)
+            ("Footnote" (make-note-footnote)))))
   (if (and (style-has? "calc-dtd")
            (not (style-has? "icourse-dtd"))
            (calc-ready?))
@@ -94,13 +107,18 @@
 
 (menu-bind insert-image-menu
   (if (and (style-has? "env-float-dtd") (in-text?))
-      ("Small figure" (make 'small-figure))
-      ("Big figure" (make 'big-figure))
-      ---)
+      (when (not (selection-active-non-small?))
+        ("Small figure"
+         (wrap-selection-small
+           (make 'small-figure)))
+        ("Big figure"
+         (wrap-selection-small
+           (insert-go-to '(big-figure "" (document "")) '(0 0))))
+        ---))
   ("Link image" (choose-file make-link-image "Load image" "image"))
   ("Insert image" (choose-file make-inline-image "Load image" "image"))
   (if (detailed-menus?)
-      ("Thumbnails" (make-thumbnails)))
+      ("Thumbnails" (interactive make-thumbnails)))
   (if (or (lazy-plugin-force)
 	  (and (style-has? "scripts-dtd") (scripts-defined? "gnuplot")))
       (-> "Plot" (link scripts-plot-menu)))
