@@ -58,10 +58,12 @@
 
 (define old-primitive-load primitive-load)
 (define (new-primitive-load filename)
-  ;; We explicitly circumvent guile's decision to set the current-reader
-  ;; to #f inside ice-9/boot-9.scm, try-module-autoload
-  (with-fluids ((current-reader read))
-               (old-primitive-load filename)))
+  (if (member (scheme-dialect) (list "guile-a" "guile-b"))
+      (old-primitive-load filename)
+      ;; We explicitly circumvent guile's decision to set the current-reader
+      ;; to #f inside ice-9/boot-9.scm, try-module-autoload
+      (with-fluids ((current-reader read))
+                   (old-primitive-load filename))))
 
 (if developer-mode?
     (begin
@@ -128,7 +130,11 @@
 (use-modules (utils library smart-table))
 (use-modules (utils plugins plugin-convert))
 (use-modules (utils misc markup-funcs))
+(use-modules (utils misc artwork))
 (use-modules (utils handwriting handwriting))
+(lazy-tmfs-handler (utils automate auto-tmfs) automate)
+(lazy-define (utils automate auto-tmfs) auto-load-help)
+(lazy-keyboard (utils automate auto-kbd) in-auto?)
 (define supports-email? (url-exists-in-path? "mmail"))
 (if supports-email? (use-modules (utils email email-tmfs)))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
@@ -156,6 +162,7 @@
 (lazy-menu (texmacs menus preferences-menu) preferences-menu page-setup-menu)
 (lazy-menu (texmacs menus preferences-widgets) open-preferences)
 (use-modules (texmacs menus main-menu))
+(lazy-define (texmacs menus file-menu) recent-file-list recent-directory-list)
 (lazy-define (texmacs menus view-menu) set-bottom-bar test-bottom-bar?)
 (tm-define (notify-set-attachment name key val) (noop))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
@@ -172,6 +179,7 @@
            page-header-menu page-footer-menu page-numbering-menu
            page-break-menu)
 (lazy-menu (generic document-menu) document-menu
+           cite-texmacs-menu cite-texmacs-short-menu
            project-menu document-style-menu global-language-menu)
 (lazy-menu (generic document-part)
            preamble-menu document-part-menu project-manage-menu)
@@ -181,12 +189,15 @@
              get-init-page-rendering init-page-rendering)
 (lazy-define (generic generic-edit) notify-activated notify-disactivated)
 (lazy-define (generic generic-doc) focus-help)
-(lazy-define (generic generic-widgets) search-toolbar replace-toolbar
+(lazy-define (generic search-widgets) search-toolbar replace-toolbar
              open-search toolbar-search-start interactive-search
              open-replace toolbar-replace-start interactive-replace
              search-next-match)
-(lazy-define (generic format-widgets) open-paragraph-format open-page-format
-             open-pattern-selector)
+(lazy-define (generic spell-widgets) spell-toolbar
+             open-spell toolbar-spell-start interactive-spell)
+(lazy-define (generic format-widgets) open-paragraph-format open-page-format)
+(lazy-define (generic pattern-selector) open-pattern-selector
+             open-gradient-selector open-background-picture-selector)
 (lazy-define (generic document-widgets) open-source-tree-preferences
              open-document-paragraph-format open-document-page-format
              open-document-metadata open-document-colors)
@@ -201,6 +212,8 @@
 (tm-property (open-document-metadata) (:interactive #t))
 (tm-property (open-document-colors) (:interactive #t))
 (tm-property (open-pattern-selector cmd w) (:interactive #t))
+(tm-property (open-gradient-selector cmd) (:interactive #t))
+(tm-property (open-background-picture-selector cmd) (:interactive #t))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 ;(display* "memory: " (texmacs-memory) " bytes\n")
 
@@ -226,6 +239,7 @@
 ;(display* "memory: " (texmacs-memory) " bytes\n")
 
 ;(display "Booting programming modes\n")
+(lazy-format (prog prog-format) cpp scheme scala java python)
 (lazy-keyboard (prog prog-kbd) in-prog?)
 (lazy-menu (prog prog-menu) prog-format-menu prog-format-icons
 	   prog-menu prog-icons)
@@ -236,12 +250,16 @@
 (lazy-keyboard (source source-kbd) always?)
 (lazy-menu (source source-menu) source-macros-menu source-menu source-icons
            source-transformational-menu source-executable-menu)
-(lazy-define (source macro-edit) has-macro-source? edit-macro-source)
-(lazy-define (source macro-widgets) editable-macro? open-macros-editor
-	     open-macro-editor create-table-macro)
-(tm-property (open-macro-editor l) (:interactive #t))
-(tm-property (create-table-macro l) (:interactive #t))
-(tm-property (open-macros-editor) (:interactive #t))
+(lazy-define (source macro-edit)
+             has-macro-source? edit-macro-source edit-focus-macro-source)
+(lazy-define (source macro-widgets)
+             editable-macro? open-macros-editor
+	     open-macro-editor create-table-macro
+             edit-focus-macro edit-previous-macro)
+(tm-property (open-macro-editor l mode) (:interactive #t))
+(tm-property (create-table-macro l mode) (:interactive #t))
+(tm-property (open-macros-editor mode) (:interactive #t))
+(tm-property (edit-focus-macro) (:interactive #t))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 ;(display* "memory: " (texmacs-memory) " bytes\n")
 
@@ -281,6 +299,12 @@
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 ;(display* "memory: " (texmacs-memory) " bytes\n")
 
+;(display "Booting educational features\n")
+(lazy-keyboard (education edu-kbd) in-edu-text?)
+(lazy-menu (education edu-menu) edu-insert-menu)
+;(display* "time: " (- (texmacs-time) boot-start) "\n")
+;(display* "memory: " (texmacs-memory) " bytes\n")
+
 ;(display "Booting dynamic features\n")
 (lazy-keyboard (dynamic fold-kbd) always?)
 (lazy-keyboard (dynamic scripts-kbd) always?)
@@ -298,6 +322,7 @@
              screens-switch-to dynamic-make-slides overlays-context?)
 (lazy-define (dynamic session-edit) scheme-eval)
 (lazy-define (dynamic calc-edit) calc-ready? calc-table-renumber)
+(lazy-define (dynamic scripts-plot) open-plots-editor)
 (lazy-initialize (dynamic session-menu) (in-session?))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 ;(display* "memory: " (texmacs-memory) " bytes\n")
@@ -309,10 +334,12 @@
 (lazy-menu (doc help-menu) help-menu)
 (lazy-define (doc tmdoc) tmdoc-expand-help tmdoc-expand-help-manual
              tmdoc-expand-this tmdoc-include)
-(lazy-define (doc docgrep) docgrep-in-doc docgrep-in-src docgrep-in-texts)
+(lazy-define (doc docgrep) docgrep-in-doc docgrep-in-src
+             docgrep-in-texts docgrep-in-recent)
 (lazy-define (doc tmdoc-search) tmdoc-search-style tmdoc-search-tag
              tmdoc-search-parameter tmdoc-search-scheme)
-(lazy-define (doc tmweb) tmweb-convert-dir tmweb-update-dir
+(lazy-define (doc tmweb) youtube-select
+             tmweb-convert-dir tmweb-update-dir
              tmweb-convert-dir-keep-texmacs tmweb-update-dir-keep-texmacs
              tmweb-interactive-build tmweb-interactive-update)
 (lazy-define (doc apidoc) apidoc-all-modules apidoc-all-symbols)
@@ -320,12 +347,12 @@
 (lazy-tmfs-handler (doc docgrep) grep)
 (lazy-tmfs-handler (doc tmdoc) help)
 (lazy-tmfs-handler (doc apidoc) apidoc)
-(define-secure-symbols tmdoc-include)
+(define-secure-symbols tmdoc-include youtube-select)
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 ;(display* "memory: " (texmacs-memory) " bytes\n")
 
 ;(display "Booting converters\n")
-(lazy-format (convert rewrite init-rewrite) texmacs scheme cpp verbatim)
+(lazy-format (convert rewrite init-rewrite) texmacs verbatim)
 (lazy-format (convert tmml init-tmml) tmml)
 (lazy-format (convert latex init-latex) latex)
 (lazy-format (convert html init-html) html)
@@ -335,6 +362,8 @@
 (lazy-define (convert images tmimage)
              export-selection-as-graphics clipboard-copy-image)
 (lazy-define (convert rewrite init-rewrite) texmacs->code texmacs->verbatim)
+(lazy-define (convert html tmhtml) ext-tmhtml-eqnarray*)
+(define-secure-symbols ext-tmhtml-eqnarray*)
 (lazy-define (convert html tmhtml-expand) tmhtml-env-patch)
 (lazy-define (convert latex latex-drd) latex-arity latex-type)
 (lazy-define (convert latex tmtex) tmtex-env-patch)
@@ -368,10 +397,11 @@
 ;(display* "memory: " (texmacs-memory) " bytes\n")
 
 ;(display "Booting security tools\n")
-(lazy-define (security wallet wallet-menu) with-wallet)
+(lazy-define (security wallet wallet-menu) expand-with-wallet)
+(tm-define-macro (with-wallet . body) (expand-with-wallet body))
 (lazy-define (security wallet wallet-base)
 	     supports-wallet? wallet-initialized?
-	     wallet-on? wallet-off?)
+	     wallet-on? wallet-off? wallet-get)
 (lazy-menu (security wallet wallet-menu) wallet-preferences-widget)
 (lazy-define (security gpg gpg-edit) tree-export-encrypted
 	     tm-gpg-dialogue-passphrase-decrypt-buffer)
@@ -399,6 +429,9 @@
              link-follow-ids)
 (lazy-define (link link-extern) get-constellation
              get-link-locations register-link-locations)
+(lazy-menu (link ref-menu) ref-menu)
+(lazy-define (link ref-edit) preview-reference)
+(define-secure-symbols preview-reference)
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 ;(display* "memory: " (texmacs-memory) " bytes\n")
 
@@ -419,6 +452,9 @@
 
 ;(display "Booting editing modes for various special styles\n")
 (lazy-menu (various poster-menu) poster-block-menu)
+(lazy-menu (various theme-menu) basic-theme-menu)
+(lazy-define (various theme-edit) current-basic-theme)
+(lazy-define (various theme-menu) basic-theme-name)
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 ;(display* "memory: " (texmacs-memory) " bytes\n")
 
@@ -434,14 +470,15 @@
 (lazy-define (fonts font-old-menu)
 	     text-font-menu math-font-menu prog-font-menu)
 (lazy-define (fonts font-new-widgets)
-             open-font-selector open-document-font-selector)
+             open-font-selector open-document-font-selector
+             open-document-other-font-selector)
 (tm-property (open-font-selector) (:interactive #t))
 (tm-property (open-document-font-selector) (:interactive #t))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 ;(display* "memory: " (texmacs-memory) " bytes\n")
 
 ;(display "Booting regression testing\n")
-(lazy-define (check check-master) check-all run-checks)
+(lazy-define (check check-master) check-all run-checks run-all-tests)
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 ;(display* "memory: " (texmacs-memory) " bytes\n")
 
