@@ -249,8 +249,17 @@
 (define (make-enum p style)
   "Make @(enum :%3 :string?) item."
   (with (tag cmd vals val width) p
-    (widget-enum (object->command (menu-protect cmd))
-                 (vals) (val) style width)))
+    (let* ((xval (val))
+           (xvals (vals))
+           (nvals (if (and (nnull? xvals) (== (cAr xvals) ""))
+                      `(,@(cDr xvals) ,xval "") `(,@xvals ,xval)))
+           (xvals* (list-remove-duplicates nvals))
+           (tval (translate xval))
+           (tvals (map translate xvals*))
+           (dec (map (lambda (v) (cons (translate v) v)) xvals*))
+           (cmd* (lambda (r) (cmd (or (assoc-ref dec r) r)))))
+      (widget-enum (object->command (menu-protect cmd*))
+                   tvals tval style width))))
 
 (define (make-choice p style)
   "Make @(choice :%3) item."
@@ -666,7 +675,7 @@
             (else
              (make-menu-items-list p style bar?)))
       (cond ((== p '---) (list (make-menu-hsep)))
-            ((== p '|) (list (make-menu-vsep)))
+            ((== p '|) (list (make-menu-vsep))) ;; '|
             ((== p '()) p)
             (else (list (make-menu-bad-format p style))))))
 
@@ -882,6 +891,15 @@
          (replace-procedures p))
         (else (menu-expand-list p))))
 
+(tm-define (cache-menu? r)
+  (:type (-> object bool))
+  (:synopsis "Cache expanded menu @r.")
+  (cond ((symbol? r) (!= r 'input))
+        ((pair? r)
+         (and (cache-menu? (car r))
+              (cache-menu? (cdr r))))
+        (else #t)))
+
 (define-table menu-expand-table
   (--- ,(lambda (p) `(--- ,@(menu-expand-list (cdr p)))))
   (| ,(lambda (p) `(| ,@(menu-expand-list (cdr p)))))
@@ -979,7 +997,7 @@
            (men (menu-promise))
            (scm (list 'vertical men))
            (wid (make-menu-widget scm 0)))
-      (alt-window-create-quit win wid name qui)
+      (alt-window-create-quit win wid (translate name) qui)
       (alt-window-show win))))
 
 (tm-define (dialogue-window menu-promise cmd name . opts)
@@ -992,7 +1010,7 @@
            (men (menu-promise lbd))
            (scm (list 'vertical men))
            (wid (make-menu-widget scm 0)))
-      (alt-window-create-quit win wid name qui)
+      (alt-window-create-quit win wid (translate name) qui)
       (alt-window-show win))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1005,7 +1023,7 @@
          (lbd (lambda x (apply cmd x) (alt-window-delete win)))
          (com (object->command (menu-protect lbd)))
          (wid (wid-promise com)))
-    (alt-window-create win wid name #t)
+    (alt-window-create-plain win wid (translate name))
     (alt-window-show win)))
 
 (tm-define (interactive-print done u)
@@ -1020,6 +1038,7 @@
 
 (tm-define (interactive-color cmd proposals)
   (:interactive #t)
+  (set! proposals (map tm->tree proposals))
   (if (not (qt-gui?))
       (interactive-rgb-picker cmd proposals)
       (with p (lambda (com) (widget-color-picker com #f proposals))
@@ -1028,6 +1047,7 @@
 
 (tm-define (interactive-background cmd proposals)
   (:interactive #t)
+  (set! proposals (map tm->tree proposals))
   (if (not (qt-gui?))
       (interactive-rgb-picker cmd proposals)
       ;;(with p (lambda (com) (widget-color-picker com #t proposals))
@@ -1078,3 +1098,8 @@
 (tm-define (show-message msg title)
   (:interactive #t)
   (dialogue-window (message-widget msg) noop title))
+
+(tm-define (restart-message)
+  (:interactive #t)
+  (show-message "Restart TeXmacs in order to let changes take effect"
+                "Notification"))
