@@ -155,6 +155,15 @@
      (ahash-set! alternate-table ',first ',second)
      (ahash-set! alternate-table ',second ',first)))
 
+(tm-define (alternate-reverse? t)
+  #f)
+
+(tm-define (alternate-first-name t)
+  (alternate-second-name t))
+
+(tm-define (alternate-first-icon t)
+  "tm_alternate_first.xpm")
+
 (tm-define (alternate-second-name t)
   "Expand")
 
@@ -316,6 +325,9 @@
   (argument-tag) (value-tag) (quote-tag)
   (binary-operation-tag) (binary-compare-tag))
 
+(define-group similar-tag
+  (label-tag) (reference-tag) (citation-tag))
+
 (define-group hidden-tag
   hidden hidden*)
 
@@ -334,5 +346,14 @@
 (define-group binary-compare-tag
   equal unequal less lesseq greater greatereq)
 
+(define-group label-tag
+  label)
+
 (define-group reference-tag
-  reference eqref pageref)
+  reference pageref eqref)
+
+(define-group citation-tag
+  cite nocite cite-TeXmacs cite-detail)
+
+(define-group mini-flow-tag
+  table graphics ornament ornamented tree)
